clc; clear all; close all
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Actuator Cylinder model %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Date of creation 05/05/2020

%% Input parameters
% Blade
B = 2;                      % Number of blades [-]
R = 1.48/2;                 % Radius [m]
c = 0.075;                  % Chord length [m]
sigma = B*c/(2*R);          % Solidity [-]
twist = deg2rad(0);         % Twist angle [rad]

% Flow conditions
Vinf = 4.0;                 % Incoming wind speed, from west to east [m/s]
rho = 1.225;                % Air density [kg/m^3]
lambda = 4.0;                 % Tip speed ratio [-]
theta_p = deg2rad(0);       % Pitch angle [rad]
omega = lambda*Vinf/R;      % Rotational speed, counter-clockwise(+) [rad/s]

% Locations
AP = 0.25*c;                % Location aerodynamic point (forces act)
MP = 0.5*c;                 % Location mounting point
EP = 0.5*c;                 % Location evaluation point
deltaAM = AP-MP;
deltaEM = EP-MP;

% Additional parameters
N = 70;                     % Number of cells in cylinder [-]
Ngrid = 10;                 % Number of cells in grid [-]
f = 1.001;                  % Factor for evaluation points [-]
tol = 1e-7;                 % Tolerance for while loop [-]
relax = 0.5;                % Relaxation paramter [-]

% Polar
load ../data/blade_optydb_Re0.082422_V3.mat
state = 1;
%POLAR(1).CL(POLAR(1).alpha>5) = POLAR(1).CL(POLAR(1).alpha==5);
%POLAR(1).CL(POLAR(1).alpha<-8) = POLAR(1).CL(POLAR(1).alpha==-8);
POLAR(1).alpha = POLAR(1).alpha;
%POLAR(2).alpha = POLAR(2).alpha+3;
POLAR(1).CL = smooth(POLAR(1).CL)*1.00;
POLAR(1).CD = smooth(POLAR(1).CD)+0.01;

%% Define control points
% Discretise rotor
dtheta = 2*pi/N;
theta = (1:N)*dtheta-1/2*dtheta;
time = theta/omega;

% Define mounting points
xm = round(-R*sin(theta),10);
ym = round(R*cos(theta),10);

% Define mounting points
xmr = round(-R*sin(theta),10)/R;
ymr = round(R*cos(theta),10)/R;

% Define aerodynamic point
xa = f*(xm+deltaAM*cos(theta))/R;
ya = f*(ym+deltaAM*sin(theta))/R;

% Define evaluation point 
xe = f*(xm+deltaEM*cos(theta))/R;
ye = f*(ym+deltaEM*sin(theta))/R;

%% Influence matrix
% For aerodynamic point
Rwx = zeros(N,N);
Rwy = zeros(N,N);

for i=1:N           % Influence of control points
    for j=1:N       % On evaluation points
        theta_int = linspace((theta(i)-0.5*dtheta),(theta(i)+0.5*dtheta),1000);
        Rx_int = (-(xa(j)+sin(theta_int)).*sin(theta_int)+(ya(j)-cos(theta_int)).*cos(theta_int))./((xa(j)+sin(theta_int)).^2+(ya(j)-cos(theta_int)).^2);
        Ry_int = (-(xa(j)+sin(theta_int)).*cos(theta_int)-(ya(j)-cos(theta_int)).*sin(theta_int))./((xa(j)+sin(theta_int)).^2+(ya(j)-cos(theta_int)).^2);
        
        Rwx(i,j) = -trapz(theta_int,Rx_int);
        Rwy(i,j) = -trapz(theta_int,Ry_int);
    end
end

% For evaluation point
Rwx_e = zeros(N,N);
Rwy_e = zeros(N,N);

for i=1:N           % Influence of control points
    for j=1:N       % On evaluation points
        theta_int = linspace((theta(i)-0.5*dtheta),(theta(i)+0.5*dtheta),1000);
        Rx_int = (-(xe(j)+sin(theta_int)).*sin(theta_int)+(ye(j)-cos(theta_int)).*cos(theta_int))./((xe(j)+sin(theta_int)).^2+(ye(j)-cos(theta_int)).^2);
        Ry_int = (-(xe(j)+sin(theta_int)).*cos(theta_int)-(ye(j)-cos(theta_int)).*sin(theta_int))./((xe(j)+sin(theta_int)).^2+(ye(j)-cos(theta_int)).^2);
        
        Rwx_e(i,j) = -trapz(theta_int,Rx_int);
        Rwy_e(i,j) = -trapz(theta_int,Ry_int);
    end
end

%% Compute induced velocities

% Prealocate
wx = zeros(1,N);
wy = zeros(1,N);
wx_e = zeros(1,N);
wy_e = zeros(1,N);
wxold = ones(1,N);
wyold = ones(1,N);
wx_i = zeros(N,N);
wy_i = zeros(N,N);

iter = 0;

while (sum(abs(wx-wxold)>tol)>=1) | (sum(abs(wy-wyold)>tol)>=1)
    % Initialise
    iter = iter+1;
    wxold = wx;
    wyold = wy;
    
    dtheta_ma = atan(((deltaAM)*cos(theta_p))./(R-(deltaAM)*sin(theta_p)));
    dtheta_me = atan(((deltaEM)*cos(theta_p))./(R-(deltaEM)*sin(theta_p)));
    
    % Calculate velocities at aerodynamic point
    Vx = Vinf+Vinf.*wx;                                                     % X-direction, same as the wind speed [m/s]
    Vy = Vinf.*wy;                                                          % Y-direction, pointing north [m/s]
    Vt = Vx.*cos(theta)+Vy.*sin(theta)+omega*R.*cos(dtheta_ma);             % Tangential, from leading to trailing edge [m/s]
    Vn = Vx.*sin(theta)-Vy.*cos(theta)+omega*R.*sin(dtheta_ma);             % Radial, pointing inwards [m/s]
    Vrel = sqrt(Vt.^2+Vn.^2);                                               % Relative velocity, magnitude [m/s]
     
    % Calculate velocities at evaluation point
    Vx_e = Vinf+Vinf.*wx_e;                                                 % X-direction, same as the wind speed [m/s]
    Vy_e = Vinf.*wy_e;                                                      % Y-direction, pointing north [m/s]
    Vt_e = Vx_e.*cos(theta)+Vy_e.*sin(theta)+omega*R.*cos(dtheta_me);       % Tangential, from leading to trailing edge [m/s]
    Vn_e = Vx_e.*sin(theta)-Vy_e.*cos(theta)+omega*R.*sin(dtheta_me);       % Radial, pointing inwards [m/s]
    Vrel_e = sqrt(Vt_e.^2+Vn_e.^2);                                         % Relative velocity, magnitude [m/s]
    
    % Calculate angle of attack at aerodynamic point
    phi = atan2(Vn,Vt);                                                     % Inflow angle [rad]
    alpha = phi-theta_p-twist;                                              % Angle of attack [rad]

    % Calculate angle of attack at evaluation point
    phi_e = atan2(Vn_e,Vt_e);                                               % Flow angle [rad]
    alpha_e = phi_e-theta_p-twist;                                          % Angle of attack [rad]
    
    % Airfoil lift and drag
    [Cl, Cd] = Dynamicstall(alpha_e,time,c,(Vrel),POLAR,N,state);          % Lift and drag with dynamic stall [-]
    %[Cl, Cd] = liftdrag(alpha_e,POLAR,state);                               % Lift and drag without dynamic stall [-]
     
    % Blade loading
    Cn = Cl.*cos(alpha)+Cd.*sin(alpha);                                     % Normal, pointing inwards (on chordline axis)   [-]                     
    Ct = Cl.*sin(alpha)-Cd.*cos(alpha);                                     % Tangential, from trailing to leading edge (on chordline axis) [-]
    
    Fn = 0.5*rho*Vrel.^2*c.*Cn;                                             % Normal, pointing inwards (on chordline axis) [N/m]
    Ft = 0.5*rho*Vrel.^2*c.*Ct;                                             % Tangential, from trailing to leading edge (on chordline axis) [N/m]

    Qn = B*(Fn.*cos(theta_p)-Ft.*sin(theta_p))./(2*pi*R*rho*Vinf^2);        % Positive outwards (on rotational axis)
    Qt = -B*(Ft.*cos(theta_p)+Fn.*sin(theta_p))./(2*pi*R*rho*Vinf^2);       % Positive trailing to leading edge (on rotational axis)
    
    % Induced velocity at aerodynamic point and evaluation point
    % Influence of control point(i) on other locations(j)
    wx = 1/(2*pi)*Qn*Rwx+1/(2*pi)*Qt*Rwy;
    wy = 1/(2*pi)*Qn*Rwy-1/(2*pi)*Qt*Rwx;
    wx_e = 1/(2*pi)*Qn*Rwx_e+1/(2*pi)*Qt*Rwy_e;
    wy_e = 1/(2*pi)*Qn*Rwy_e-1/(2*pi)*Qt*Rwx_e;    
       
    % Add effect of cylinder
    if f >= 1                                                               % In the wake
        for j = N/2+1:N
            wx(j)   = wx(j)  +Qn(j)-Qn(N+1-j)-Qt(j)*(ymr(j)/sqrt(1-ymr(j)^2))-Qt(N+1-j)*(ymr(j)/sqrt(1-ymr(j)^2));
            wx_e(j) = wx_e(j)+Qn(j)-Qn(N+1-j)-Qt(j)*(ymr(j)/sqrt(1-ymr(j)^2))-Qt(N+1-j)*(ymr(j)/sqrt(1-ymr(j)^2));
        end    
    elseif f < 1                                                            % Inside the cylinder
        for j = 1:N/2
            wx(j) = wx(j)-Qn(j)-Qt(j)*(ymr(j)/sqrt(1-ymr(j)^2));
            wx_e(j) = wx_e(j)-Qn(j)-Qt(j)*(ymr(j)/sqrt(1-ymr(j)^2));

        end
        for j = N/2+1:N
            wx(j) = wx(j)-Qn(N+1-j)-Qt(N+1-j)*(ymr(j)/sqrt(1-ymr(j)^2));
            wx_e(j) = wx_e(j)-Qn(N+1-j)-Qt(N+1-j)*(ymr(j)/sqrt(1-ymr(j)^2));
        end
    end
    
    % Mod-Lin model
    CT = dtheta*sum(Qn.*sin(theta) + Qt.*cos(theta));
    
    %%% method 1
    a = 0.0892074*CT^3+0.0544955*CT^2+0.251163*CT-0.0017077;
    ka = abs(1/(1-a));
    
    if a<=0.15
        ka = abs(1/(1-a));
    else
        ka = abs(1/(1-a))*(0.65+0.35*exp(-4.5*(a-0.15)));
    end
   
    wxnew = ka.*wx;
    wynew = ka.*wy;

    % Relaxation
    wx = ((1-relax).*wxold+relax.*wxnew);
    wy = ((1-relax).*wyold+relax.*wynew);
    
    wx(wx>0.5) = 0.5;

      
    wx_e = ka.*wx_e;
    wy_e = ka.*wy_e;
     
    if iter>200
        warning('More than 200 iterations are required. The code is stopped')
        break
    end

end


%% Power coefficient
Cp_ideal=trapz(theta,Qn.*Vn./(Vinf)); 
Cp_real=1/(2*pi)*trapz(theta,B.*((Ft.*cos(theta_p)+Fn.*sin(theta_p)).*omega.*R))/(0.5*rho*Vinf^3*2*R);

Cp_local = -Qt.*lambda;
Cp = (sum(Cp_local*dtheta));

CT = dtheta*sum(Qn.*sin(theta) + Qt.*cos(theta));
CT2 = trapz(theta,Qn.*sin(theta)+Qt.*cos(theta));

disp(['Power coefficeint, CP: ' num2str(Cp)]);
disp(['Thrust coefficeint, CT: ' num2str(CT)]);

%% Make thrust variations
FN_B1 = (Fn.*cos(theta_p)-Ft.*sin(theta_p))*1.5; % positive is pointing inwards (on rotational axis)
FT_B1 = (Ft.*cos(theta_p)+Fn.*sin(theta_p))*1.5; % positive is pointing from trailing to leading edge (on rotational axis)
FX_B1 = FN_B1.*sin(theta)- FT_B1.*cos(theta); % positive is poiting in streamwise direction
FX_B2 = [FX_B1(N/2+1:end) FX_B1(1:N/2)];

FX_turbine = FX_B1+FX_B2;

%% Save data
ACmodel_DS_flowcurv_V2.alpha = rad2deg(alpha);
ACmodel_DS_flowcurv_V2.theta = rad2deg(theta);
ACmodel_DS_flowcurv_V2.Vrel = Vrel;
ACmodel_DS_flowcurv_V2.CT = CT;
ACmodel_DS_flowcurv_V2.Fn_blade1 = FN_B1;
ACmodel_DS_flowcurv_V2.Ft_blade1 = FT_B1;
ACmodel_DS_flowcurv_V2.Qn_blade1 = Qn;
ACmodel_DS_flowcurv_V2.Qt_blade1 = Qt;
ACmodel_DS_flowcurv_V2.wx = wx;
ACmodel_DS_flowcurv_V2.wy = wy;
ACmodel_DS_flowcurv_V2.Fx_blade1 = FX_B1;
ACmodel_DS_flowcurv_V2.Fx_blade2 = FX_B2;
ACmodel_DS_flowcurv_V2.T = FX_turbine;

cd('../data')
filename = 'ACmodel_DS_flowcurv';
save([filename '.mat' ],'ACmodel_DS_flowcurv')
%% Figures
figure(1)
    subplot(2,3,1)
    grid on
    hold on
    plot(theta*180/pi,alpha*180/pi,'-x')
    xlabel('theta [deg]')
    ylabel('alpha [deg]')
    
    subplot(2,3,2)
    grid on
    hold on
    plot(theta*180/pi,Vrel,'-x')
    xlabel('theta [deg]')
    ylabel('Vrel [m/s]')
    
    subplot(2,3,3)
    grid on
    hold on
    plot(theta*180/pi,Qn,'-x')
    xlabel('theta [deg]')
    ylabel('Qn [-]')
    
    subplot(2,3,4)
    grid on
    hold on
    plot(theta*180/pi,Qt,'-x')
    xlabel('theta [deg]')
    ylabel('Qt [-]')    
    
    subplot(2,3,5)
    grid on
    hold on
    plot(theta*180/pi,wx,'-x')
    xlabel('theta [deg]')
    ylabel('wx [-]')      
    
    subplot(2,3,6)
    grid on
    hold on
    plot(theta*180/pi,wy,'-x')
    xlabel('theta [deg]')
    ylabel('wy [-]')     
    
