%    function  SPL=TBLnoise_BPM_Xfoil(alpha,L,c,U,f,DH,DL,r,deltastar_p,deltastar_s)
   function  SPL=TBLnoise_BPM_Xfoil(alpha,L,c,U,f,DH,DL,r)

  
% Calculates the dB SPL at one metre from the trailing edge
% From the three components of trailing edge noise
% The code follows brooks and pope

% U: flow speed relative to the blade
% c: chord length
% L: segment length
% alpha: angle of attack /degrees
% f : frequency /Hz
% tripped : tripping boolean 

c0=342.943;       %sound speed [m/s]
rho=1.20615;      %density [kg/m3]
nu=1.50142e-5;     %kinematic viscosity [m2/s]


M=U/c0;        %Mach number [-] where U is the blade velocity [m/s]
Re=U.*c/nu;    %chord Reynolds number where c is the chord length of the blade [m]

% Compute the boundary layer thickness, the boundary layer displacement
% thickness and momentum thickness

    if Re <= 0.3*10^6
        
        delta_0=c.*10.^(1.892-0.9045*log10(Re)+0.0596*(log10(Re).^2));
        deltastar_0=c.*0.0601.*Re^-0.114;
        
    else
        
        delta_0=c.*10.^(1.892-0.9045*log10(Re)+0.0596*(log10(Re).^2));
        deltastar_0=c.*10.^(3.411-1.5397*log10(Re)+0.1059*(log10(Re).^2));
    end

% Compute the boundary layer thickness for the pressure side

    delta_p=delta_0.*10.^(-0.04175*alpha+0.00106*alpha.^2);
    deltastar_p=deltastar_0.*10.^(-0.0432*alpha+0.00113*alpha.^2);
    
% Compute the boundary layer thickness for the suction side

if (alpha >= 0) && (alpha <= 5)
    delta_s =delta_0.*10.^(0.0311*alpha);
    deltastar_s = delta_0.*10.^(0.0679*alpha);
    
elseif (alpha > 5) && (alpha <= 12.5)
    delta_s=delta_0.*0.3468.*10.^(0.1231.*alpha);
    deltastar_s = delta_0.*0.381*10.^(0.1516*alpha);
else 
   delta_s=delta_0.*5.718.*10.^(0.0258.*alpha);
   deltastar_s = deltastar_0.*14.296.*10.^(0.0258.*alpha);
end




% for the A spectral model

St_1=0.02*M.^(-0.6);
if alpha < 1.33
   St_2 = St_1.* 1;
elseif (alpha >= 1.33) && (alpha <= 12.5)
    St_2 = St_1.*10^(0.0054.*(alpha-1.33)^2);
else
    St_2 = St_1.*4.72;
end

St_p=f.*deltastar_p./U;
St_s=f.*deltastar_s./U;

% K_1 model

if Re < 2.47 *10^5
    K_1 = -4.31.*log10(Re) + 156.3;
elseif (Re >= 2.47*10^5) && (Re <= 8.0*10^5)
    K_1 = -9.0.*log10(Re) + 181.6;
else
    K_1 = 128.5;
end

% DeltaK_1

Redeltastar_p=U.*deltastar_p/nu;

if Redeltastar_p <= 5000
    DeltaK_1 = alpha.*(1.43.*log10(Redeltastar_p) - 5.29);
else
    DeltaK_1 = 0;
end

% K_2 model

gamma=27.094*M+3.31;
beta=72.65*M+10.74;
gamma0=23.43*M+4.651;
beta0=-34.19*M-13.82;

if alpha < (gamma0-gamma)
   K_2=K_1+(-1000);
elseif (alpha >= (gamma0 - gamma)) && alpha <= (gamma0+gamma)
        K_2 = K_1+ sqrt(beta.^(2) - (beta/gamma).^2*(alpha-gamma0).^2) + beta0;
else
    K_2 = K_1 - 12;
end


% Main formula 
St_1_bar=(St_1+St_2)/2;


if alpha < 12.5 
    SPL_p=10*log10(deltastar_p.*M.^5.*L*DH./r^2)+A(St_p,St_1,Re)+(K_1-3)+DeltaK_1;
    SPL_s=10*log10(deltastar_s.*M.^5.*L*DH./r^2)+A(St_s,St_1_bar,Re)+(K_1-3);
    SPL_alpha=10*log10(deltastar_s.*M.^5.*L*DH./r^2)+B(St_s,St_2,Re)+K_2;

elseif alpha < gamma0
% main formula only valid for  alpha<12.5 and alpha<gamma0

SPL_p=10*log10(deltastar_p.*M.^5.*L*DH./r^2)+A(St_p,St_1,Re)+(K_1-3)+DeltaK_1;
SPL_s=10*log10(deltastar_s.*M.^5.*L*DH./r^2)+A(St_s,St_1_bar,Re)+(K_1-3);
SPL_alpha=10*log10(deltastar_s.*M.^5.*L*DH./r^2)+B(St_s,St_2,Re)+K_2;

else
% Now for alpha>12.5 or alpha>gamma0

%The contribution from the pressure side and from the suction side go to
%infinity.
%A value in this case corresponds to A'. It is equal to A but now the
%Reynolds number is 3Re.

SPL_p=-inf;
SPL_s=-inf;
SPL_alpha=10*log10(deltastar_s.*M.^5.*L*DL./r^2)+A(St_s,St_2,3*Re)+K_2;

end


% Add them all up

SPL=10*log10(10.^(SPL_p/10)+10.^(SPL_s/10)+10.^(SPL_alpha/10));

% 
% figure(12121)
%   hold on
%   grid on
%  semilogx(f,SPL_p,'b-v');
%  semilogx(f,SPL_s,'r-^');
%  semilogx(f,SPL_alpha,'g');
%  semilogx(f,SPL,'c');
%  axis([0 10^4 -20 60]);
%  xlabel('Frequency [Hz]','interpreter','latex','FontSize',30,'FontWeight','bold');
%  ylabel('SPL [dB]','interpreter','latex','FontSize',30,'FontWeight','bold');
%  lg = legend('$SPL_{p}$','$SPL_{s}$','$SPL_{alpha}$','$SPL_{tot}$');
%  set(lg,'Interpreter','latex');
%  set(gca,'FontSize',28,'TickLabelInterpreter','latex');

  
