function [SPL] = TNO(U1,x2,tke,L1,f,L,R,suction,rho,D,varargin)

%TNO method
% this function computes the noise of an airfoil based on the TNO method
% http://orbit.dtu.dk/ws/files/3309574/ris-r-1633.pdf

%inputs are:
%U1, boundary layer velocity vector
%x2, boundary layer height vector
%tke, tke vector along BL
%L1, length scale vector (NOTE: in this case the turbulence length scale
%corresponds to 2Lt2 and is calculated! It is the turbulence length scale in
%the turbulent boundary layer!!).
%omeg, frequency vector (omeg=2pif)
%L,spanwise extent of the trailing edge (divided section)
%R, distance to receiver
%rho, density
%D directivity ,blade.directivityH

%         ./\
%         |
%         |
%         | x2

% LE ------- TE     x1-->

%% Preallocating values
F=zeros(length(U1),length(f)); 
G=zeros(length(U1),length(f)); 
H=zeros(length(U1),length(f)); 
gamma_p=zeros(length(U1),length(f)); 
sig_22=zeros(length(U1),length(f));

%% Reshape all data
f=reshape(f,length(f(:)),1);
U1=reshape(U1,length(U1(:)),1);
x2=reshape(x2,length(x2(:)),1);
tke=reshape(tke,length(tke(:)),1);
L1=reshape(L1,length(L1(:)),1);


%% Constants
%sound speed [m/s]
c0=340.3;

%freestream velocity [m/s]. Because U1 is the velocity profile in the
%boundary layer and it corresponds to U(y) = 0.99U_inf.
U0= max(U1)./0.99;

%Turbulence length scale along x2, Lt2. Assuming an homogeneous, isotropic
%turbulence then the turbulence length scale at inflow (Lt1) and the one in
%the boundary layer (Lt2) are related as: Lt1 = 2Lt2. The one that it is an
%input is 2Lt2 if I want Lt2 than I have to divide by 2.
L2=L1/2;                     

%spanwise wavenumber. Observer is located in the mid-span.The wavenumber in the spanwise
%direction is ignored.
k3=0;                   

%Constant c_alpha [-]
c4=0.7;
% c4=0.65;
%Convection velocity is a function of the local boundary layer velocity U1
%page 24 TNO paper DTU.
Uc=c4.*U1;  

% Velocity gradient
delU1 = gradient(U1,abs(x2(1)-x2(2))); 

%wavenumber of the main energy bearing eddies (ke) [1/m]
%(3/4 = 0.7468(equals to the sqrt(pi)*gamma(5/6)/gamma(1/3))and 
%len_scale is equal to Lt2). This Equation shifts teh peak frequency
%towards a higher frequency.
ke=0.7468./(L2);  
% ke=0.7468./(2*L2);  


if suction==1 %alfa constant changes for non symmetric airfoils
    alf=0.45;
else %pressureside
    alf=0.3;
end

for ii=1:length(f) %loop through all frequencies
    
    omeg=2.*pi.*f(ii); %angular frequency
    k1=omeg./Uc;   %non-constant wavenumber
    k=sqrt(k1.^2+k3.^2); %in the end k = k1;
    
    %turbulent shear stress (proportional to TKE). Also called the root
    %mean square value of the vertical velocity fluctuations. (Equation 25,
    %TNO model DTU).
    u22 = alf.*tke;
    
    %Gaussian constant (a_g) is a function of the eddy convection velocity and the turbulent length scale.
    a_g=0.05.*Uc./L2;
    
    %Moving axis spectrum (sig_m) describes how the vertical velocity
    %spectrum (sig_22) is distorted by the generation and the destruction
    %of eddies during their convection past the trailing edge.(page 23 TNO
    %model, DTU)
    sig_m =(1./(a_g.*sqrt(pi))).*exp(-((omeg-Uc.*k1)./a_g));
    
    %Vertical velocity spectrum (sig_22). Remember that k3 = 0.
    sig_22(:,ii) = (4.*((k1./ke).^2)./(9.*pi.*ke.^2.*(1+(k1./ke).^2).^(7./3)));   
    
    %F
    F(:,ii)= 4.*rho.^2.*k1.^2./k.^2.*(L2).*delU1.^2.*u22;
    
    %G is the vertical velocity spectrum that determines the noise peak
    %frequency.
    G(:,ii)= sig_22(:,ii).*sig_m;
    
    %H is a simple exponential decay function that can be viewed as a
    %weighting function.
    H(:,ii)= exp(-2.*x2.*abs(k1));
    
    int_2=(F(:,ii).*G(:,ii).*H(:,ii));
     
     %Remember that trapz is the trapezoidal numerical integration. Z =
     %trapz(X,Y) computes the inetragral of Y with respect to X using the
     %trapezoidal method.
     %In this case is the integral of the product between F, G and H
     %along x2.
     %gamma_p is the surface pressure wavenumber-frequency spectral density
     gamma_p(:,ii)= trapz(x2,int_2);
%      gamma_p(:,ii)=trapz(x2(2:end),int_2(2:end));
     
      %Far-field acoustic pressure spectrum density can be obtained
      %from the integral of the wall pressure spectrum over the wavenumber
      %component in the flow direction. (S_w is the PSD)
%     S_w(:,ii)=(L./(4.*pi.*R.^2)).*trapz(k1(2:end),omeg./(c0.*k1(2:end)).*gamma_p(ii)).*D;
    S_w(:,ii)=(L./(4.*pi.*R.^2)).*trapz(k1,omeg./(c0.*k1).*gamma_p(:,ii)).*D;
     
end
% S_w=thirdOctave(S_w,f);

%To convert the double-sided spectrum to the single-sided spectrum and to
%account for 1Hz narrow bandwidth, the PSD (S_w) is multiplied with an
%amplification factor of 2. (Page6/28 iTNO model paper).
amp_factor=2;
pref=(20*10^-6).^2;
%To convert the PSD in SPL.
W_f=10*log10(((amp_factor*S_w)./pref));

SPL = real(W_f);
% 
%  figure(299919)
%  hold on
%  semilogx(f,SPL,'-b')
%  grid on
%  axis([0 10^4 -20 60]);
%  xlabel('Frequency [Hz]','interpreter','latex','FontSize',30,'FontWeight','bold');
%  ylabel('SPL [dB]','interpreter','latex','FontSize',30,'FontWeight','bold');
%  set(gca,'FontSize',28,'TickLabelInterpreter','latex');
% % %TNO plots
%
% %normalised probe fig1b
%   plot(U1./max(U1),x2/max(x2))
% % %gradient fig1c
%    plot(delU1,x2/max(x2))
% % %length scale fig1d
% figure(23)
%   plot(L1,x2/max(x2))
%   hold on
% % %tke normalised fig1e
%  plot(tke./max(U1).^2,x2/max(x2))
% % %shear stress fig1f
%   plot(u22,x2/max(x2))
% % %ke fig1g
%   plot(ke,x2/max(x2))
% % %k1/ke
%   plot(k1./ke,k1.^2.*sig22(ii))
%
% %
% % %
