function [SPL] = iTNO(U1,x2,tke,L1,f,L,R,suction,rho,D,varargin)

%iTNO method   
% this function computes the noise of an airfoil based on the ITNO method
% (Seongkyu Lee, 2016)

%inputs are:
%U1, boundary layer velocity vector
%x2, boundary layer height vector
%tke, tke vector along BL
%L1, length scale vector.(NOTE: in this case the turbulence length scale
%corresponds to 2Lt2 and is calculated! It is the turbulence length scale in
%the turbulent boundary layer!!).
%omeg, frequency vector (omeg=2pif)
%L,spanwise extent of the trailing edge (divided section)
%R, distance to receiver
%rho, density
%D directivity ,blade.directivityH

%         ./\
%         |
%         |
%         | x2

% LE ------- TE     x1-->

% for iTNO k1 is a constant with x but depends on the angular frequency for
% G and H.

%% Preallocating values
if length(varargin) == 1
    u22 =varargin{1}';
elseif length(varargin)>1
    u22 = varargin{1}';
end


F=zeros(length(U1),length(f));
G=zeros(length(U1),length(f)); 
H=zeros(length(U1),length(f));
gamma_p=zeros(length(U1),length(f));
sig_22=zeros(length(U1),length(f));

%% Reshape all data
f=reshape(f,length(f(:)),1);
U1=reshape(U1,length(U1(:)),1);
x2=reshape(x2,length(x2(:)),1);
tke=reshape(tke,length(tke(:)),1);
L1=reshape(L1,length(L1(:)),1);
u22=reshape(u22,length(u22(:)),1);


%% Constants
%sound speed [m/s]
c0=340.3;

%freestream velocity [m/s]. Because U1 is the velocity profile in the
%boundary layer and it corresponds to U(y) = 0.99U_inf.
U0= max(U1)./0.99;

%Turbulence length scale along x2, Lt2. Assuming an homogeneous, isotropic
%turbulence then the turbulence length scale at inflow (Lt1) and the one in
%the boundary layer (Lt2) are related as: Lt1 = 2Lt2. The one that it is an
%input is 2Lt2 if I want Lt2 than I have to divide by 2.
L2=L1/2;   

%Anisotropy factor
beta1=1;                
beta3=0.75;

%spanwise wavenumber. Observer is located in the mid-span.The wavenumber in the spanwise
%direction is ignored.
k3=0;

%Constant c_alpha [-]. Page 9/28 iTNO model. Use this value to avoid
%singularity problem.
c4=0.65;
%Convection velocity is a function of the local boundary layer velocity U1
%page 24 TNO paper DTU.
Uc=c4.*U1;  

% Velocity gradient
delU1 = gradient(U1,abs(x2(1)-x2(2)));

%wavenumber of the main energy bearing eddies (ke) [1/m]
%(3/4 = 0.7468(equals to the sqrt(pi)*gamma(5/6)/gamma(1/3))and 
%len_scale is equal to 2Lt2). Equation 30 iTNO model. It provides a better
%agreement with the measurement data for the peak frequency.
ke=0.7468./(2.*L2);

%F takes into account the source region beneath the boundary layer. It is 
%only a function of the vertical distance, x2. In the iTNO model, F is 
%indipendent from the angular frequency. (While in TNO model, it depends on
%both variables). Equation 39 iTNO model.
F= 4.*rho.^2.*(L2).*delU1.^2.*u22./U0;


for ii=1:length(f) %loop through all frequencies
    
    omeg=2.*pi.*f(ii); %angular frequency
    k1= omeg./Uc;  %it is indipendent of x2.
    k=sqrt(k1.^2+k3.^2);
   
    %vertical velocity spectrum (Equation 32-iTNO model). The dependence on omega is already taken
    %into account with k (see equation above).
    sig_22(:,ii) = (4.*beta1.*beta3.*((beta1.*k./ke).^2)./(9.*pi.*ke.^2.*(1+(beta1.*k./ke).^2).^(7./3)));
    
   %in the iTNO model, no moving-axis spectrum sig_m. It is assumed a frozen
   %turbulence, where turbulence is assumed to be convected unchanged as it
   %passes over the trailing edge. This assumption makes the moving axis
   %spectrum a simple Dirac delta function.
   
  %Equation 40 iTNO model.
    G(:,ii)= sig_22(:,ii);
  
  %Equation 41 iTNO model.
    H(:,ii)= exp(-2.*x2.*abs(k1));
    
    %integrate of boundary layer velocity profile. Equation 37 and 39 iTNO
    %model.
    gamma_p= trapz(x2,F.*G(:,ii).*H(:,ii));
    
    %Inside the integral of Equation 1 iTNO model.
    int2(:,ii)=(omeg./(c0.*k1)).*gamma_p;
    
end
% 
% 
%     
% figure(23123)
% semilogx(f,k1)
% hold on
% semilogx(f,H)
% semilogx(f,G)

%Since k1 is constant do not perform any integration. Equation 1 iTNO
%model.
S_w=((L./(4.*pi.*R.^2)).*int2).*D; 
pref=(20*10^-6)^2;
% S_w=thirdOctave(S_w,f);
amp_factor=4*pi;
%S_w is PSD
W_f=10*log10(((amp_factor*S_w)./pref));
SPL=real(W_f);

% figure(23123)
%  hold on
%  semilogx(f,SPL,'-b')
%  grid on
%  axis([0 10^4 -20 60]);
%  xlabel('Frequency [Hz]','interpreter','latex','FontSize',30,'FontWeight','bold');
%  ylabel('SPL [dB]','interpreter','latex','FontSize',30,'FontWeight','bold');
%  set(gca,'FontSize',28,'TickLabelInterpreter','latex');



% 
% if suction==0
%     
%     
%     figure(121212)
%      [max_f,I1]=max(max(G));
%      
%      [~,I2]=(max(SPL));
%      hold on
%      scatter(I1,I2,'k')
% 
% 
% end

% 
% 
 
  
  % max(SPL)
  
  
% 
%
% correct params:

%  x2 in trapz
%  pref^2 - Lee paper
%  10log - Lee paper


 %figure(271)
%hold on
%semilogx(f,SPL)
% hold on
%
%
% if max(u22)>4
%    SPL=zeros(1,32);
%   a=1
% end
%
%
%
% if SPL(27)>5
%
% figure(9)
% disp(['>>>>>>SUCTION=  ' num2str(suction)])
% hold on



%fig 1g
% figure(10)
% plot(ke,x2/max(x2))
% hold on

%fig 1a
% figure(11)
% plot(U1./max(U1),x2/max(x2))
% hold on
%     SPL=W_f.*0;
%
% end


%fig 1h
 %semilogx(k1./ke,k1.^2.*sig_22)

%
%
% %
%
%
%
% %TNO plots
% %

%
% % f(ij);
% %
% %
% % % %
%    [f1,d1]=meshgrid(f,x2./max(x2));
%     figure (23)
%     contourf(f1,d1,real(F*ones(1,32).*1.*1))
%        contourf(f1,d1,real(G))
%               surf(f1,d1,real(H))
% 
%         axis ([0 10000 0 1])
% 
%     colormap jet
%     caxis([0 4*10^-7])
%     
%            surf(f1,d1,real(G))


%
%     a=1;
%     
%     
%     %%
%     
%     
% 
%         contourf(f1,d1,real(10*log10(delU1.^2)*ones(1,32).*1.*1))
%                 caxis([50 80])
% 
%                 contourf(f1,d1,real(L2*ones(1,32).*1.*1))
% 
%     colormap jet

