function [noise] = inflowNoise(blade,count,len_scale,intensity,noise,f,ij)

%This function gives the inflow noise for the two formulations:
%1) Paterson and Amiet. The incoming turbulence is defined using the Von
%Karman spectrum.
%2) Amiet original formulation.
%3) Buck. He suggests that Kolomogrov and Von karman turbulence specra are
%quivalent at large length scales encountered by large wind turbines.

%For 1) and 3) formulations, the high and low-frequency components of noise are
%combined using a blending funciton as proposed by Lowson and Moriarity and
%Migliore.

% NOTE: the directivity function used in the model is the one for the
% leading edge because the inflow-turbulence noise is radiated from the LE.

%% Parameters

%inflow wavenumber (k) [1/m]
Kx=2.*pi*f./blade.W;

%Mach number correction (Beta 2 = beta squared) [-]
Beta2=1-blade.M.^2;

%kc/2 used in LFC and S
Kbar=Kx*blade.chord./2;

%wavenumber of the main energy bearing eddies (ke) [1/m]
%(3/4 = 0.7468(equals to the sqrt(pi)*gamma(5/6)/gamma(1/3))and 
%len_scale is equal to Lt1).
Ke=3/(4*len_scale);

%ratio between inflow wavenumber and ke adopted in the main formula.
Khat=Kx/Ke;

%Approximation of the compressible Sears function (S2 = Ssquared) [-]
S2=(2*pi*Kbar./Beta2+(1+2.4*Kbar./Beta2).^(-1)).^(-1); 

%Low Frequency Correction factor (LFC) [-]
LFC=10.*S2.*(1+9.*deg2rad(blade.aoa).^2).*blade.M.*(Kbar).^2.*Beta2.^-1;

%% SPLs
try
%Paterson and Amiet formulation
SPL.H=10*log10(((blade.rho.^2.*blade.c0.^2.*blade.ddiv.*len_scale.*blade.M.^3.*(intensity.*blade.W).^2).*(Khat.^3./(1+Khat.^2).^(7/3)).*blade.directivityL_TI)./(2.*blade.re.^2))+78.4;
noise.SPL.inflow(ij,:,count)=SPL.H+10*log10(LFC./(1+LFC));
noise.SPL.inflow_temp=noise.SPL.inflow(ij,:,count); %this one is used for the contour plot
catch
    a=1;
end
%Amiet original formulation
SPL.amiet=10*log10((len_scale.*blade.ddiv.*blade.M.^5.*intensity.^2.*((Khat.^3)./(1+Khat.^2).^(7/3))).*blade.directivityL_TI./blade.re.^2)+181.3;
noise.SPL.amiet(ij,:,count)=SPL.amiet;
noise.SPL.amiet_temp=noise.SPL.amiet(ij,:,count); %this one is used for the contour plot

%Buck formulation
SPL.H_buck=10*log10((blade.rho.^2.*blade.c0.^2.*blade.ddiv.*blade.M.^3.*noise.eta.^(2/3).*Kx.^(-5/3).*blade.directivityL_TI)./(2.*blade.re.^2))+77.6;
noise.SPL.buck(ij,:,count)=SPL.H_buck+10*log10(LFC./(1+LFC));
noise.SPL.buck_temp=noise.SPL.buck(ij,:,count); %this one is used for the contour plot

%% Find sound power
% http://en.wikipedia.org/wiki/Sound_power
noise.SPL.inflow_power(ij,:,count)=noise.SPL.inflow(ij,:,count) - 10*log10(1/(4*pi*blade.re.^2));
          
% %
%  figure(912)
%  grid on
%  semilogx(f,noise.SPL.inflow_temp,'-b')
%  hold on
%  semilogx(f,noise.SPL.amiet_temp,'-g')
%  hold on
%  semilogx(f, noise.SPL.buck_temp,'-r')
%  xlabel('Frequency [Hz]','interpreter','latex','FontSize',30,'FontWeight','bold');
%  ylabel('SPL [dB]','interpreter','latex','FontSize',30,'FontWeight','bold');
%  lg = legend('$SPL_{Paterson}$','$SPL_{Amiet}$','$SPL_{Buck}$');
%  set(lg,'Interpreter','latex');
%  set(gca,'FontSize',28,'TickLabelInterpreter','latex');

